/* mysql2csv - Write out the result of a MySQL query as a CSV file
 *  Copyright (C) 2000  Simon Richter
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <string.h>

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#if (!defined(HAVE_LIBMYSQLCLIENT)) || (!defined(HAVE_MYSQL_MYSQL_H))
#error You need MySQL installed to use this program
#endif

#include <mysql/mysql.h>

#define ERROR(x) fprintf(stderr, "%s:%d: %s\n", __FILE__, __LINE__, x)
#define DBERR(x) fprintf(stderr, " -> %s\n", mysql_error(x))

void usage() {
	printf("Usage: mysql2csv [-h host] [-u user] [-p password] db query\n");
	return;
}

int main(int argc, char *argv[]) {
	MYSQL temp, *db;
	MYSQL_RES *res;

	MYSQL_ROW r;

	char *host = NULL;
	char *user = NULL;
	char *password = NULL;

	char buffer[1024];

	int t;

	while((t = getopt(argc, argv, "h:u:p:")) != -1)
		switch(t) {
			case 'h':
				host = optarg;
				break;
			case 'u':
				user = optarg;
				break;
			case 'p':
				password = optarg;
				break;
			default:
				ERROR("Unexpected answer from getopt()");
		}

	argc -= optind;
	argv += optind;

	if(argc < 2) {
		usage();
		exit(1);
	}

	buffer[0] = '\0';
	for(t = 1; t < argc-1; t++) {
		strncat(buffer, argv[t], 1024);
		strncat(buffer, " ", 1024);
	}
	strncat(buffer, argv[t], 1024);

	if(! mysql_init(&temp)) {
		ERROR("mysql_init() failed");
		exit(1);
	}

	db = mysql_real_connect(&temp,
			host,
			user,
			password,
			argv[0],
			3306,
			NULL,
			0);
	if(! db) {
		ERROR("mysql_real_connect failed");
		DBERR(&temp);
		exit(1);
	}

	if(mysql_query(db, buffer)) {
		ERROR("mysql_query() failed");
		DBERR(db);
		goto closedb;
	}

	res = mysql_store_result(db);
	if(! res) {
		ERROR("mysql_store_result() failed");
		DBERR(db);
		goto closedb;
	}

	for(t = 0; t < (int)res->field_count-1; t++)
		printf("%s;", res->fields[t].name);
	printf("%s\n", res->fields[t].name);

	while((r = mysql_fetch_row(res))) {
		for(t = 0; t < (int)res->field_count-1; t++)
			printf("%s;", r[t]?r[t]:"");
		printf("%s\n", r[t]?r[t]:"");
	}
	mysql_free_result(res);
closedb:
	mysql_close(db);
	exit(0);
}

